uab.classes.ClickActivator = uab.mixins.aceOO.Class()

function uab.classes.ClickActivator.prototype:init(name, data)
	uab.classes.ClickActivator.super.prototype.init(self)

	self.name = name
	self:SetActivationData(data)
	self.groups = {}

    uab.GroupIterate(
        function(self, context, name, info)
            if not context.groups[info.name] then
                context.groups[info.name] = {}
            end
            self.groups[info.name] = uab.classes.UnitGroup:new(self, info.name, info.unitcount, context.groups[info.name])
        end,
        self,
        data)	

	if data.enabled then
	    self:Enable()
	else
	    self:Disable()
	end
end

function uab.classes.ClickActivator.prototype:CopyTo(target, fromgroup, togroup)
    --[[
    A deep copy, not including the name or activation click combination.
    Basically, copy the group data/actions for all groups
    --]]

    uab.GroupIterate(
        function(self, context, name, info)
            if not context.fromgroup then
                self.groups[info.name]:CopyTo(context.target.groups[info.name])
            elseif context.fromgroup == info.name then
                if not context.togroup then
                    context.togroup = context.fromgroup
                end
                self.groups[info.name]:CopyTo(context.target.groups[context.togroup])
                return true
            end
        end,
        self,
        { fromgroup = fromgroup, target = target, togroup = togroup} )
end

function uab.classes.ClickActivator.prototype:IAmDead()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:IAmDead()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:SetAction(group, number, data)
	if self.groups[group] then
	    self.groups[group]:SetAction(number, data)
	else
	    -- error : unknown group name
	end
end

function uab.classes.ClickActivator.prototype:ClearAction(group, number)
	if self.groups[group] then
	    self.groups[group]:ClearAction(number)
	else
	    -- error : unknown group name
	end
end

function uab.classes.ClickActivator.prototype:SetLayout(group, layout)
    uab.GroupIterate(
        function(self, context, name, info)
            if not context.group or context.group == info.name then
                self.groups[info.name]:SetLayout(context.layout)
                if context.group then
                    return true
                end
            end
        end,
        self,
        { group = group, layout = layout} )
end

function uab.classes.ClickActivator.prototype:Enable()
	self.data.enabled = true
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:Enable()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:Disable()
	self.data.enabled = nil
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:Disable()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:Enabled()
	return self.data.enabled
end

function uab.classes.ClickActivator.prototype:TimerEvent(elapsed)
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:TimerEvent(elapsed)
        end,
        self)
end

function uab.classes.ClickActivator.prototype:RegisterFrame(frame)
	--[[
	Determine the correct group based on the unit associated with the frame.
	If no unit associated with the frame, then use the "partyraid" group.
	--]]

    local unit = SecureButton_GetModifiedUnit(frame)
    local group
    
    uab.pdebug("Registering frame : '"..frame:GetName().."' in activator "..self.name)

    if unit then
        if unit == "player" or unit == "pet" then
            group = self.groups[unit]
        elseif "target" == string.sub(unit, 1, 6) then
            group = self.groups[unit]
        elseif "focus" == string.sub(unit, 1, 5) then
            group = self.groups[unit]
        else
            group = self.groups.partyraid
        end
    end
    
    if not group then
        group = self.groups.partyraid
    end

    group:RegisterFrame(frame)
end

function uab.classes.ClickActivator.prototype:ScaleChanged(groupname)
    uab.GroupIterate(
        function(self, context, name, info)
            if not context or context == info.name then
                self.groups[info.name]:ScaleChanged()
                if context then
                    return true
                end
            end
        end,
        self,
        groupname)
end

function uab.classes.ClickActivator.prototype:PositionChanged(groupname)
    uab.GroupIterate(
        function(self, context, name, info)
            if not context or context == info.name then
                self.groups[info.name]:PositionChanged()
                if context then
                    return true
                end
            end
        end,
        self,
        groupname)
end

function uab.classes.ClickActivator.prototype:DirectCastChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:DirectCastChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:StayOpenChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:StayOpenChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:FastChangeUnitChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:FastChangeUnitChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:ResetButtons()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:ResetButtons()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:ClearActivationData()
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.ClickActivator.prototype:SetActivationData(data)
    if data then
		self.data = data
	end
    
	if self.data.button == "LeftButton" then
		self.buttoncode = "1"
	elseif self.data.button == "RightButton" then
		self.buttoncode = "2"
	elseif self.data.button == "MiddleButton" then
		self.buttoncode = "3"
	elseif self.data.button == "Button4" then
		self.buttoncode = "4"
	else
		self.buttoncode = "5"
	end

    self.prefix = ""
    
	if self.data.prefixshift then
		self.prefix = "shift-" .. self.prefix
	end
	
	if self.data.prefixctrl then
		self.prefix = "ctrl-" .. self.prefix
	end
	
	if self.data.prefixalt then
		self.prefix = "alt-" .. self.prefix
	end
end
